@@ -7,7 +7,7 @@
 * packets, or reads from dump file.
 *
 *  -F format (names can be abbreviated):
-*     dump     dump in binary format
+*     dump     dump in binary format (suitable for rtpplay)
 *     header   like 'dump', but don't save audio/video payload
 *     ascii    parsed packets (default)
 *     hex      like 'ascii', but with hex dump
@@ -140,6 +140,12 @@ static int open_network(char *host, int data, int sock[], struct
                    sizeof(one)) == -1)
       perror("setsockopt: reuseaddr");
 
+#ifdef SO_REUSEPORT /* BSD 4.4 */
+    if (setsockopt(sock[i], SOL_SOCKET, SO_REUSEPORT, (char *) &one,
+		   sizeof(one)) == -1)
+      perror("setsockopt: reuseport");
+#endif
+
     sin->sin_port = htons(ntohs(sin->sin_port) + i);
     if (bind(sock[i], (struct sockaddr *)sin, sizeof(*sin)) < 0) {
       if (errno == EADDRNOTAVAIL) {
@@ -155,7 +161,7 @@ static int open_network(char *host, int data, int sock[], struct
       }
     }
 
-    if (IN_CLASSD(mreq.imr_multiaddr.s_addr)) {
+    if (IN_CLASSD(ntohl(mreq.imr_multiaddr.s_addr))) {
       if (setsockopt(sock[i], IPPROTO_IP,
         IP_ADD_MEMBERSHIP, (char *)&mreq, sizeof(mreq)) < 0) {
           perror("IP_ADD_MEMBERSHIP");
@@ -250,10 +256,10 @@ static int parse_data(char *buf, int len)
       return hlen;
     }
     printf(
-    "v=%d p=%d x=%d cc=%d m=%d pt=%d (%s,%d,%d) seq=%lu ts=%lu ssrc=0x%lx\n",
+    "v=%d p=%d x=%d cc=%d m=%d pt=%d (%s,%d,%d) seq=%u ts=%lu ssrc=0x%lx\n",
       r->version, r->p, r->x, r->cc, r->m, 
       r->pt, pt_map[r->pt].enc, pt_map[r->pt].ch, pt_map[r->pt].rate,
-      ntohl(r->seq), ntohl(r->ts), ntohl(r->ssrc));
+      ntohs(r->seq), ntohl(r->ts), ntohl(r->ssrc));
     for (i = 0; i < r->cc; i++) {
       printf("csrc[%d] = %0x ", i, r->csrc[i]);
     }
@@ -346,7 +352,7 @@ static char *rtp_read_sdes(char *b, int len)
   }
   b = (char *)rsp + 1;
   /* skip padding */
-  return b + ((4 - ((long)b & 0x3)) & 0x3);
+  return b + ((4 - ((int)b & 0x3)) & 0x3);
 } /* rtp_read_sdes */
 
 
@@ -369,17 +375,19 @@ static int parse_control(char *buf, int len)
   else if (r->common.version == RTP_VERSION) {
     printf("\n");
     while (len > 0) {
-      len -= (r->common.length + 1) << 2;
+      len -= (ntohs(r->common.length) + 1) << 2;
       if (len < 0) {
         /* something wrong with packet format */
-        printf("Illegal RTCP packet length %d words.\n", r->common.length);
+        printf("Illegal RTCP packet length %d words.\n",
+	       ntohs(r->common.length));
         return -1;
       }
 
       switch (r->common.pt) {
       case RTCP_SR:
-        printf(" (SR ssrc=0x%x p=%d count=%d len=%d\n", 
-          (int)r->r.rr.ssrc, r->common.p, r->common.count, r->common.length);
+        printf(" (SR ssrc=0x%lx p=%d count=%d len=%d\n", 
+          ntohl(r->r.rr.ssrc), r->common.p, r->common.count,
+	       ntohs(r->common.length));
         printf("ntp=%lu.%lu ts=%lu psent=%lu osent=%lu\n",
           ntohl(r->r.sr.ntp_sec),
           ntohl(r->r.sr.ntp_frac),
@@ -390,7 +398,7 @@ static int parse_control(char *buf, int len)
           printf("  (ssrc=%0lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu)\n",
            ntohl(r->r.sr.rr[i].ssrc),
            r->r.sr.rr[i].fraction / 256.,
-           ntohl(r->r.sr.rr[i].lost),
+           ntohl(r->r.sr.rr[i].lost), /* XXX I'm pretty sure this is wrong */
            ntohl(r->r.sr.rr[i].last_seq),
            ntohl(r->r.sr.rr[i].jitter),
            ntohl(r->r.sr.rr[i].lsr),
@@ -400,8 +408,9 @@ static int parse_control(char *buf, int len)
         break;
 
       case RTCP_RR:
-        printf(" (RR ssrc=0x%x p=%d count=%d len=%d\n", 
-          (int)r->r.rr.ssrc, r->common.p, r->common.count, r->common.length);
+        printf(" (RR ssrc=0x%lx p=%d count=%d len=%d\n", 
+          ntohl(r->r.rr.ssrc), r->common.p, r->common.count,
+	       ntohs(r->common.length));
         for (i = 0; i < r->common.count; i++) {
           printf("(ssrc=%0lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu)\n",
            ntohl(r->r.rr.rr[i].ssrc),
@@ -417,15 +426,16 @@ static int parse_control(char *buf, int len)
 
       case RTCP_SDES:
         printf(" (SDES p=%d count=%d len=%d\n", 
-          r->common.p, r->common.count, r->common.length);
+          r->common.p, r->common.count, ntohs(r->common.length));
         buf = (char *)&r->r.sdes;
         for (i = 0; i < r->common.count; i++) {
-          int remaining = (r->common.length << 2) - (buf - (char *)&r->r.sdes);
+          int remaining = (ntohs(r->common.length) << 2) -
+	    (buf - (char *)&r->r.sdes);
 
-          printf("  (src=0x%x ", ((struct rtcp_sdes *)buf)->src);
+          printf("  (src=0x%lx ", ntohl(((struct rtcp_sdes *)buf)->src));
           if (remaining > 0) {
             buf = rtp_read_sdes(buf, 
-              (r->common.length << 2) - (buf - (char *)&r->r.sdes));
+              (ntohs(r->common.length) << 2) - (buf - (char *)&r->r.sdes));
             if (!buf) return -1;
           }
           else {
@@ -439,11 +449,11 @@ static int parse_control(char *buf, int len)
 
       case RTCP_BYE:
         printf(" (BYE p=%d count=%d len=%d\n", 
-          r->common.p, r->common.count, r->common.length);
+          r->common.p, r->common.count, ntohs(r->common.length));
         for (i = 0; i < r->common.count; i++) {
-          printf("ssrc[%d]=%0x ", i, r->r.bye.src[i]);
+          printf("ssrc[%d]=%0lx ", i, ntohl(r->r.bye.src[i]));
         }
-        if (r->common.length > r->common.count) {
+        if (ntohs(r->common.length) > r->common.count) {
           buf = (char *)&r->r.bye.src[r->common.count];
           printf("reason=\"%*.*s\"", *buf, *buf, buf+1); 
         }
@@ -452,10 +462,10 @@ static int parse_control(char *buf, int len)
 
       /* invalid type */
       default:
-        printf("(? pt=%d src=0x%x)\n", r->common.pt, (int)r->r.sdes.src);
+        printf("(? pt=%d src=0x%lx)\n", r->common.pt, ntohl(r->r.sdes.src));
       break;
       }
-      r = (rtcp_t *)((u_int32 *)r + r->common.length + 1);
+      r = (rtcp_t *)((u_int32 *)r + ntohs(r->common.length) + 1);
     }
   }
   else {
@@ -520,7 +530,7 @@ void packet_handler(t_format format, int trunc, double dstart, struct
       case F_ascii:
         printf("%8ld.%06ld %s len=%d from=%s:%u ",
             now.tv_sec, now.tv_usec, parse_type(ctrl, packet.p.data),
-            len, inet_ntoa(sin.sin_addr), sin.sin_port);
+            len, inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
         if (format == F_hex) {
           hex(packet.p.data, trunc < len ? trunc : len);
         }
@@ -619,24 +629,36 @@ int main(int argc, char *argv[])
     pt_map[i].rate = 0;
     pt_map[i].ch   = 0;
   }
+  /* Updated 25 Jul 1997 to reflect IANA assignments: */
+  /* <ftp://ftp.isi.edu/in-notes/iana/assignments/rtp-av-payload-types> */
+  /* Marked items are indicated as 'reserved' by the IANA */
   pt_map[  0].enc = "PCMU"; pt_map[  0].rate =  8000; pt_map[  0].ch = 1;
   pt_map[  1].enc = "1016"; pt_map[  1].rate =  8000; pt_map[  1].ch = 1;
   pt_map[  2].enc = "G721"; pt_map[  2].rate =  8000; pt_map[  2].ch = 1;
   pt_map[  3].enc = "GSM "; pt_map[  3].rate =  8000; pt_map[  3].ch = 1;
   pt_map[  4].enc = "G723"; pt_map[  4].rate =  8000; pt_map[  4].ch = 1;
-  pt_map[  5].enc = "IDVI"; pt_map[  5].rate =  8000; pt_map[  5].ch = 1;
-  pt_map[  6].enc = "IDVI"; pt_map[  6].rate = 16000; pt_map[  6].ch = 1;
+  pt_map[  5].enc = "DVI4"; pt_map[  5].rate =  8000; pt_map[  5].ch = 1;
+  pt_map[  6].enc = "DVI4"; pt_map[  6].rate = 16000; pt_map[  6].ch = 1;
+  pt_map[  7].enc = "LPC "; pt_map[  7].rate =  8000; pt_map[  7].ch = 1;
+  pt_map[  8].enc = "PCMA"; pt_map[  8].rate =  8000; pt_map[  7].ch = 1;
+  pt_map[  9].enc = "G722"; pt_map[  9].rate =  8000; pt_map[  7].ch = 1;
   pt_map[ 10].enc = "L16 "; pt_map[ 10].rate = 44100; pt_map[ 10].ch = 2;
   pt_map[ 11].enc = "L16 "; pt_map[ 11].rate = 44100; pt_map[ 11].ch = 1;
-  pt_map[ 23].enc = "SCR "; pt_map[ 23].rate = 65536; pt_map[ 23].ch = 0;
-  pt_map[ 24].enc = "MPEG"; pt_map[ 24].rate = 65536; pt_map[ 24].ch = 0;
-  pt_map[ 25].enc = "CELB"; pt_map[ 25].rate = 65536; pt_map[ 26].ch = 0;
-  pt_map[ 26].enc = "JPEG"; pt_map[ 26].rate = 65536; pt_map[ 26].ch = 0;
-  pt_map[ 27].enc = "CUSM"; pt_map[ 27].rate = 65536; pt_map[ 27].ch = 0;
-  pt_map[ 28].enc = "NV  "; pt_map[ 28].rate = 65536; pt_map[ 28].ch = 0;
-  pt_map[ 29].enc = "PicW"; pt_map[ 29].rate = 65536; pt_map[ 29].ch = 0;
-  pt_map[ 30].enc = "CPV "; pt_map[ 30].rate = 65536; pt_map[ 30].ch = 0;
-  pt_map[ 31].enc = "H261"; pt_map[ 31].rate = 65536; pt_map[ 31].ch = 0;
+  pt_map[ 14].enc = "MPA "; pt_map[ 14].rate = 90000; pt_map[ 14].ch = 0;
+  pt_map[ 15].enc = "G728"; pt_map[ 15].rate =  8000; pt_map[ 15].ch = 1;
+  pt_map[ 16].enc = "DVI4"; pt_map[ 16].rate = 11025; pt_map[ 16].ch = 1;
+  pt_map[ 17].enc = "DVI4"; pt_map[ 17].rate = 22050; pt_map[ 17].ch = 1;
+  pt_map[ 23].enc = "SCR "; pt_map[ 23].rate = 90000; pt_map[ 23].ch = 0; /**/
+  pt_map[ 24].enc = "MPEG"; pt_map[ 24].rate = 90000; pt_map[ 24].ch = 0; /**/
+  pt_map[ 25].enc = "CelB"; pt_map[ 25].rate = 90000; pt_map[ 26].ch = 0;
+  pt_map[ 26].enc = "JPEG"; pt_map[ 26].rate = 90000; pt_map[ 26].ch = 0;
+  pt_map[ 27].enc = "CUSM"; pt_map[ 27].rate = 90000; pt_map[ 27].ch = 0; /**/
+  pt_map[ 28].enc = "nv  "; pt_map[ 28].rate = 90000; pt_map[ 28].ch = 0;
+  pt_map[ 29].enc = "PicW"; pt_map[ 29].rate = 90000; pt_map[ 29].ch = 0; /**/
+  pt_map[ 30].enc = "CPV "; pt_map[ 30].rate = 90000; pt_map[ 30].ch = 0; /**/
+  pt_map[ 31].enc = "H261"; pt_map[ 31].rate = 90000; pt_map[ 31].ch = 0;
+  pt_map[ 32].enc = "MPV "; pt_map[ 32].rate = 90000; pt_map[ 32].ch = 0;
+  pt_map[ 33].enc = "MP2T"; pt_map[ 33].rate = 90000; pt_map[ 33].ch = 0;
 
   /* set maximum time to gather packets */
   timeout.tv_usec = 0;
@@ -709,6 +731,8 @@ int main(int argc, char *argv[])
       dnow = tdbl(&now);
       /* plen>0: data =0: control */
       i = (packet.p.hdr.plen == 0);
+      /* arbitrary, obviously invalid value */
+      sin.sin_addr.s_addr = INADDR_ANY; sin.sin_port = 0;
       packet_handler(format, trunc, dstart, now, i, sin, len, packet);
     }
   }
