@@ -47,7 +47,7 @@ static double period[128] = {  /* ms per timestamp difference */
   1/8000.,   /*  1: 1016 */
   1/8000.,   /*  2: G721 */
   1/8000.,   /*  3: GSM  */
-  1/8000.,   /*  4: ?    */
+  1/8000.,   /*  4: G723 */
   1/8000.,   /*  5: DVI4 */
   1/16000.,  /*  6: DVI4 */
   1/8000.,   /*  7: LPC  */
@@ -59,14 +59,15 @@ static double period[128] = {  /* ms per timestamp difference */
   0,         /* 13:      */
   1/90000.,  /* 14: MPA  */
   1/90000.,  /* 15: G728 */
-  0,         /* 16:      */
-  0,         /* 17:      */
+  1/11025.,  /* 16: DVI4 */
+  1/22050.,  /* 17: DVI4 */
   0,         /* 18:      */
   0,         /* 19:      */
   0,         /* 20:      */
   0,         /* 21:      */
   0,         /* 22:      */
   0,         /* 23:      */
+  0,         /* 24:      */
   1/90000.,  /* 25: CelB */
   1/90000.,  /* 26: JPEG */  
   1/90000.,  /* 27:      */  
@@ -76,6 +77,7 @@ static double period[128] = {  /* ms per timestamp difference */
   1/90000.,  /* 31: H261 */  
   1/90000.,  /* 32: MPV  */  
   1/90000.,  /* 33: MP2T */  
+  1/90000.,  /* 34: H263 */
 };
 
 
@@ -308,7 +310,7 @@ int main(int argc, char *argv[])
         exit(1);
       }
 
-      if (IN_CLASSD(sin.sin_addr.s_addr) && 
+      if (IN_CLASSD(ntohl(sin.sin_addr.s_addr)) && 
           (setsockopt(sock[i], IPPROTO_IP, IP_MULTICAST_TTL, &ttl,
                    sizeof(ttl)) < 0)) {
         perror("IP_MULTICAST_TTL");
